#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ostdb.h"

/*
*******************************************************************************
*
* Function: UpCase
*
* Purpose: Converts an input string to uppercase
*
* History:
* 08/07/2007  58886  MSwam    handle empty string
*
*******************************************************************************
*/

char *UpCase(char *str){
  
  char *ptr;
  
  ptr = str;

  for (ptr=str; ptr!= NULL && *ptr!='\0'; ptr++){
      *ptr = toupper(*ptr);
  }

  return(str); 
}


/*
*******************************************************************************
*
* Function: LowCase
*
* Purpose: Converts the input string to lowercase
*
* History:
* 08/07/2007  58886  MSwam    handle empty string
*
*******************************************************************************
*/

char *LowCase(char *str){
  
  char *ptr;
  
  ptr = str;

  for (ptr=str; ptr!=NULL && *ptr!='\0'; ptr++){
      *ptr = tolower(*ptr);
  }

  return(str); 
}


/*
 ******************************************************************************
 *
 * Function:     QuotedField
 *
 * Purpose:      Determines if the value of a field needs to be quoted when
 *               being inserted into the database or being used in a where
 *               clause. Datatypes that need to be quoted are char, varchar, 
 *               datetime and text. The determinatiopn is made by querying the
 *               database for the datatype of the field. 
 *
 * Returns:      TRUE or FALSE
 *
 * History:
 * 04/02/2008  MSwam  As of Sybase15, multiple records come back from the query
 *                    and we were getting the wrong answer when only some
 *                    of the types were flagged as needing quotes.  From now
 *                    on, if ANY of the types needs quotes then that 
 *                    sets our return value = TRUE.
 *
 ******************************************************************************
*/
int QuotedField(char *server, 
                char *db, 
                char *table,  
                char *column)
{
  char type[32];
  int needQuotes;

  ostdb_query_setup(STDB_CONN_3, "QuotedField", 2, STDB_SQL, server, db, NULL);
  stdb_bind_string(STDB_CONN_3, "type", type, 0);
  stdb_string_param(STDB_CONN_3, "table", table);
  stdb_string_param(STDB_CONN_3, "column", column);
  stdb_query(STDB_CONN_3, "select t.name type from systypes t,");
  stdb_query(STDB_CONN_3, "syscolumns c where c.id = object_id(@table)");
  stdb_query(STDB_CONN_3, "and c.name = @column and c.type=t.type");

  needQuotes = FALSE;
  while(ostdb_get_exec(STDB_CONN_3) == STDB_CONTINUE){
    /* if ANY of the hits need quotes, then we will return TRUE */
    if ((strstr(type, "char") != NULL) ||
        (strcmp(type, "text") == 0) ||
        (strncmp(type, "datetim", 7) == 0)){
      needQuotes = TRUE;
    }
  }
  stdb_close_query(STDB_CONN_3);
  return(needQuotes);
}

/*
*******************************************************************************
*
* Function: NotBlank
*
* Purpose: Returns FALSE if a string is NULL or has nothing but white space
*          or punctuation.
*
*******************************************************************************
*/

int NotBlank(char *str){
  
  char *cptr;

  if ((str == NULL) || (str[0] == '\0'))
    return FALSE;
  
  for (cptr=str; *cptr !='\0'; cptr++){
    if (isalnum(*cptr))
      return TRUE;
  }

  return FALSE; 
}

/*
*******************************************************************************
*
* Function: TransformQuotes
*
* Purpose: Returns a NEW allocated string that replaces any single occurrences
*          of quote (') or quotemark (") with DOUBLE occurrences of those
*          characters ('') ("").  This is needed so that some SQL engines
*          can use the value in an INPUT statement.
*
*          NOTE: The CALLER must free the memory for the outstr when no longer
*                needed.
*
* History:
* 08/07/2007  58886  MSwam    handle empty string
* 03/28/2011  67901  MSwam    change to make a COPY of the string and replace
*                               and single quotes or quotemarks with DOUBLE
*                               instances of quotes or quotemarks
*
*******************************************************************************
*/

void TransformQuotes(char *instr, char **outstr){

  char *cptr_IN, *cptr_OUT;
  int need_LARGER;  /* flag=1 if a LARGER output string is needed */
  int instr_size;    /* size of input string */

  /* Check for any occurrences of single quote or quotemark, and if ANY are
   * found, allocate a LARGER output string, so DOUBLE replacements can be
   * made.  If no occurrences are found, then just allocate an output string
   * of the SAME size as the input string.
   */
  instr_size = strlen(instr);
  need_LARGER = 0;
  if (strstr(instr, "'") != NULL) need_LARGER =1;
  if (strstr(instr, "\"") != NULL) need_LARGER =1;

  if (need_LARGER==1) {
     *outstr = (char *)malloc(instr_size * 2 + 1);  /* DOUBLE the size  */
  }
  else { /* no increase needed, use SAME size output string, make a copy,
          * and RETURN
          */
     *outstr = (char *)malloc(instr_size+1);
     strcpy(*outstr, instr);
     return;
  }

  /* step through instr, copying each char to outstr, but replacing each
   * occurrence of single quote or quotemark with TWO quote or quotemarks
   */
  for (cptr_IN=instr, cptr_OUT=*outstr;
       cptr_IN !=NULL && *cptr_IN!='\0';
       cptr_IN++)
  {
      if (*cptr_IN == '"') { /* replace single quotemark with TWO quotemarks */
         *cptr_OUT = '"'; cptr_OUT++;
         *cptr_OUT = '"'; cptr_OUT++;
      }
      else if (*cptr_IN == '\'') { /* replace single quote with TWO quotes */
         *cptr_OUT = '\''; cptr_OUT++;
         *cptr_OUT = '\''; cptr_OUT++;
      }
      else {                /* just copy along the in char to the out */
         *cptr_OUT = *cptr_IN; cptr_OUT++;
      }
  }
  *cptr_OUT = '\0';  /* assign EOS */
  return;
}
